@@ -8,8 +8,8 @@ class PluginTest(CliTestCase):
         Load plugins/foo.py as a plugin and execute its code
         """
         plugin_dir = Path(__file__).parent.joinpath('plugins').resolve()
-        result = self.cli('--plugins', str(plugin_dir), 'record', 'tests', '--session', 'dummy', 'foo', 'alpha', 'bravo', 'charlie')
+        result = self.cli('--plugins', str(plugin_dir), 'record', 'tests',
+                          '--session', '/intake/organizations/org/workspaces/work/builds/dummy/test_sessions/123', 'foo', 'alpha', 'bravo', 'charlie')
         self.assertTrue("foo:alpha" in result.stdout, result.stdout)
         self.assertTrue("foo:bravo" in result.stdout, result.stdout)
         self.assertTrue("foo:charlie" in result.stdout, result.stdout)
-
