@@ -1,5 +1,5 @@
 from pathlib import Path
-import responses # type: ignore
+import responses  # type: ignore
 import json
 import gzip
 import sys
@@ -12,9 +12,10 @@ class RspecTest(CliTestCase):
 
     @responses.activate
     def test_record_test_rspec(self):
-        result = self.cli('record', 'tests',  '--session', self.session, 'rspec', str(self.test_files_dir.joinpath("rspec.xml")))
+        result = self.cli('record', 'tests',  '--session', self.session,
+                          'rspec', str(self.test_files_dir.joinpath("rspec.xml")))
         self.assertEqual(result.exit_code, 0)
         payload = json.loads(gzip.decompress(
-            b''.join(responses.calls[0].request.body)).decode())
+            b''.join(responses.calls[1].request.body)).decode())
         expected = self.load_json_from_file(self.result_file_path)
         self.assert_json_orderless_equal(expected, payload)
